/*
<H2>Licence Agreement</H2>
<PRE>InterfaceClassFinder package
Copyright (c) 2009 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.

3. Neither the name of the author nor Forward Computing and Control may be
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment:
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd.
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
</PRE>
 */
package au.com.forward.interfaceClassFinder;

/**
 *
 * @author Matthew
 * package protected
 */
class ClassInfo {

    private final String className;  // fully qualified class name
    private final String superClassName; // fully qualified super class name
    private final String[] interfaceNames; // fully qualified array of interfaceNames
    private String source = ""; // the source never null
    private boolean hasNoArgConstructor;
    private boolean isInterfaceClass = true;
    // set to true if this class implements the required interface
    // null if has not been checked yet
    private Boolean implementsInterface;
    private boolean publicClass;
    private boolean abstractClass;

    ClassInfo(String className, String superClassName, String[] interfaceNames, boolean publicClass, boolean abstractClass) {
        if (className == null) {
            throw new IllegalArgumentException("className cannot be null.");
        }
        if (superClassName == null) {
            throw new IllegalArgumentException("superClassName cannot be null.");
        }
        if (interfaceNames == null) {
            throw new IllegalArgumentException("interfaceNames cannot be null.");
        }
        this.className = className;
        this.superClassName = superClassName;
        this.interfaceNames = interfaceNames;
        this.publicClass = publicClass;
        this.abstractClass = abstractClass;
    }

    public String getClassName() {
        return className;
    }

    public boolean isAbstractClass() {
        return abstractClass;
    }

    public boolean isPublicClass() {
        return publicClass;
    }

    public String[] getInterfaceNames() {
        return interfaceNames;
    }

    public String getSuperClassName() {
        return superClassName;
    }

    Boolean implementsInterface() {
        return implementsInterface;
    }

    void setImplementsInterface(boolean implementsInterface) {
        this.implementsInterface = implementsInterface;
    }

    void clearImplementsInterface() {
        this.implementsInterface = null;
    }
    
    boolean isInterface() {
        return isInterfaceClass;
    }

    void setIsInterface(boolean isInterfaceClass) {
        this.isInterfaceClass = isInterfaceClass;
    }

    boolean hasNoArgConstructor() {
        return hasNoArgConstructor;
    }

    void setHasNoArgConstructor(boolean hasNoArgConstructor) {
        if (hasNoArgConstructor) {
            isInterfaceClass = false;
        }
        this.hasNoArgConstructor = hasNoArgConstructor;
    }

    String getSource() {
        return source;
    }

    void setSource(String source) {
        if (source == null) {
            throw new IllegalArgumentException("source cannot be null.");
        }
        this.source = source;
    }

    String asString() {
        StringBuilder rtn = new StringBuilder();
        if (isInterfaceClass) {
            rtn.append("Interface: ");
        } else {
            rtn.append("Class: ");
        }
        rtn.append("Name:").append(this.className).append(" extends:").append(this.superClassName).append(" interfaces:");
        if (interfaceNames.length == 0) {
            rtn.append("none ");
        } else {
            for (String s : interfaceNames) {
                rtn.append(s).append(" ");
            }
        }
        rtn.append("Source:").append(this.source);
        if (!isInterfaceClass) {
            rtn.append(" hasNoArgConstructor:").append(hasNoArgConstructor);
        }
        return rtn.toString();
    }
}
